@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2002-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -36,7 +36,6 @@
 #include <config.h>
 
 #include <pthread.h>
-#include <assert.h>
 #include <pwd.h>
 #include <grp.h>
 #include <sys/types.h>
@@ -60,8 +59,8 @@
 
 #include <corosync/swab.h>
 #include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
 #include <corosync/list.h>
-#include <corosync/queue.h>
 #include <corosync/lcr/lcr_ifact.h>
 #include <corosync/totem/coropoll.h>
 #include <corosync/totem/totempg.h>
@@ -72,13 +71,13 @@
 
 #include "poll.h"
 #include "totemsrp.h"
-#include "mempool.h"
 #include "mainconfig.h"
 #include "totemconfig.h"
 #include "main.h"
 #include "sync.h"
 #include "tlist.h"
 #include "util.h"
+#include "timer.h"
 
 #define SERVER_BACKLOG 5
 
@@ -90,6 +89,8 @@ static pthread_attr_t thread_attr;
 
 static struct timerlist timers_timerlist;
 
+static int sched_priority = 0;
+
 static void (*timer_serialize_lock_fn) (void);
 
 static void (*timer_serialize_unlock_fn) (void);
@@ -106,17 +107,17 @@ static void *prioritized_timer_thread (void *data)
 	int fds;
 	unsigned long long timeout;
 
-#if ! defined(TS_CLASS) && (defined(COROSYNC_BSD) || defined(COROSYNC_LINUX) || defined(COROSYNC_SOLARIS))
-	struct sched_param sched_param;
-	int res;
+#if defined(HAVE_PTHREAD_SETSCHEDPARAM) && defined(HAVE_SCHED_GET_PRIORITY_MAX)
+	if (sched_priority != 0) {
+		struct sched_param sched_param;
 
-	sched_param.sched_priority = 2;
-	res = pthread_setschedparam (expiry_thread, SCHED_RR, &sched_param);
+		sched_param.sched_priority = sched_priority;
+		pthread_setschedparam (expiry_thread, SCHED_RR, &sched_param);
+	}
 #endif
 
 	pthread_mutex_unlock (&timer_mutex);
 	for (;;) {
-retry_poll:
 		timer_serialize_lock_fn ();
 		timeout = timerlist_msec_duration_to_expire (&timers_timerlist);
 		if (timeout != -1 && timeout > 0xFFFFFFFF) {
@@ -124,19 +125,20 @@ retry_poll:
 		}
 		timer_serialize_unlock_fn ();
 		fds = poll (NULL, 0, timeout);
-		if (fds == -1) {
-			goto retry_poll;
+		if (fds < 0 && errno == EINTR) {
+			continue;
+		}
+		if (fds < 0) {
+			return NULL;
 		}
 		pthread_mutex_lock (&timer_mutex);
 		timer_serialize_lock_fn ();
 
 		timerlist_expire (&timers_timerlist);
-		
+
 		timer_serialize_unlock_fn ();
 		pthread_mutex_unlock (&timer_mutex);
 	}
-
-	pthread_exit (0);
 }
 
 static void sigusr1_handler (int num) {
@@ -148,12 +150,14 @@ static void sigusr1_handler (int num) {
 
 int corosync_timer_init (
         void (*serialize_lock_fn) (void),
-        void (*serialize_unlock_fn) (void))
+        void (*serialize_unlock_fn) (void),
+	int sched_priority_in)
 {
 	int res;
 
 	timer_serialize_lock_fn = serialize_lock_fn;
 	timer_serialize_unlock_fn = serialize_unlock_fn;
+	sched_priority = sched_priority_in;
 
 	timerlist_init (&timers_timerlist);
 
@@ -234,11 +238,11 @@ int corosync_timer_add_duration (
 }
 
 void corosync_timer_delete (
-	timer_handle timer_handle)
+	timer_handle th)
 {
 	int unlock;
 
-	if (timer_handle == 0) {
+	if (th == 0) {
 		return;
 	}
 
@@ -249,7 +253,7 @@ void corosync_timer_delete (
 		pthread_mutex_lock (&timer_mutex);
 	}
 
-	timerlist_del (&timers_timerlist, timer_handle);
+	timerlist_del (&timers_timerlist, th);
 
 	if (unlock) {
 		pthread_mutex_unlock (&timer_mutex);
@@ -272,12 +276,12 @@ unsigned long long corosync_timer_time_get (void)
 }
 
 unsigned long long corosync_timer_expire_time_get (
-	timer_handle timer_handle)
+	timer_handle th)
 {
 	int unlock;
 	unsigned long long expire;
 
-	if (timer_handle == 0) {
+	if (th == 0) {
 		return (0);
 	}
 
@@ -288,7 +292,7 @@ unsigned long long corosync_timer_expire_time_get (
 		pthread_mutex_lock (&timer_mutex);
 	}
 
-	expire = timerlist_expire_time (&timers_timerlist, timer_handle);
+	expire = timerlist_expire_time (&timers_timerlist, th);
 
 	if (unlock) {
 		pthread_mutex_unlock (&timer_mutex);
