@@ -153,7 +153,9 @@ static unsigned short mcast_packed_msg_lens[FRAME_SIZE_MAX];
 
 static int mcast_packed_msg_count = 0;
 
-static int totempg_reserved = 0;
+static int totempg_reserved = 1;
+
+static unsigned int totempg_size_limit;
 
 /*
  * Function and data used to log messages
@@ -173,6 +175,8 @@ static void (*totempg_log_printf) (
 
 struct totem_config *totempg_totem_config;
 
+static totempg_stats_t totempg_stats;
+
 enum throw_away_mode {
 	THROW_AWAY_INACTIVE,
 	THROW_AWAY_ACTIVE
@@ -719,9 +723,12 @@ int totempg_initialize (
 		return (-1);
 	}
 
+	totemsrp_net_mtu_adjust (totem_config);
+
 	res = totemmrp_initialize (
 		poll_handle,
 		totem_config,
+		&totempg_stats,
 		totempg_deliver_fn,
 		totempg_confchg_fn);
 
@@ -732,7 +739,9 @@ int totempg_initialize (
 		callback_token_received_fn,
 		0);
 
-	totemsrp_net_mtu_adjust (totem_config);
+	totempg_size_limit = (totemmrp_avail() - 1) *
+		(totempg_totem_config->net_mtu -
+		sizeof (struct totempg_mcast) - 16);
 
 	return (res);
 }
@@ -764,7 +773,7 @@ static int mcast_msg (
 	int total_size = 0;
 
 	pthread_mutex_lock (&mcast_msg_mutex);
-	totemmrp_new_msg_signal ();
+	totemmrp_event_signal (TOTEM_EVENT_NEW_MSG, 1);
 
 	/*
 	 * Remove zero length iovectors from the list
@@ -791,7 +800,7 @@ static int mcast_msg (
 	}
 
 	if (byte_count_send_ok (total_size + sizeof(unsigned short) *
-		(mcast_packed_msg_count+1)) == 0) {
+		(mcast_packed_msg_count)) == 0) {
 
 		pthread_mutex_unlock (&mcast_msg_mutex);
 		return(-1);
@@ -872,6 +881,9 @@ static int mcast_msg (
 			iovecs[2].iov_len = max_packet_size;
 			assert (totemmrp_avail() > 0);
 			res = totemmrp_mcast (iovecs, 3, guarantee);
+			if (res == -1) {
+				goto error_exit;
+			}
 
 			/*
 			 * Recalculate counts and indexes for the next.
@@ -907,6 +919,7 @@ static int mcast_msg (
 			mcast_packed_msg_count++;
 	}
 
+error_exit:
 	pthread_mutex_unlock (&mcast_msg_mutex);
 	return (res);
 }
@@ -919,9 +932,9 @@ static int msg_count_send_ok (
 {
 	int avail = 0;
 
-	avail = totemmrp_avail () - totempg_reserved - 1;
+	avail = totemmrp_avail ();
 
-	return (avail > msg_count);
+	return ((avail - totempg_reserved) > msg_count);
 }
 
 static int byte_count_send_ok (
@@ -930,11 +943,11 @@ static int byte_count_send_ok (
 	unsigned int msg_count = 0;
 	int avail = 0;
 
-	avail = totemmrp_avail () - 1;
+	avail = totemmrp_avail ();
 
-	msg_count = (byte_count / (totempg_totem_config->net_mtu - 25)) + 1;
+	msg_count = (byte_count / (totempg_totem_config->net_mtu - sizeof (struct totempg_mcast) - 16)) + 1;
 
-	return (avail > msg_count);
+	return (avail >= msg_count);
 }
 
 static int send_reserve (
@@ -942,7 +955,7 @@ static int send_reserve (
 {
 	unsigned int msg_count = 0;
 
-	msg_count = (msg_size / (totempg_totem_config->net_mtu - 25)) + 1;
+	msg_count = (msg_size / (totempg_totem_config->net_mtu - sizeof (struct totempg_mcast) - 16)) + 1;
 	totempg_reserved += msg_count;
 
 	return (msg_count);
@@ -1163,6 +1176,10 @@ int totempg_groups_joined_reserve (
 	for (i = 0; i < iov_len; i++) {
 		size += iovec[i].iov_len;
 	}
+	if (size >= totempg_size_limit) {
+		reserved = -1;
+		goto error_put;
+	}
 
 	reserved = send_reserve (size);
 	if (msg_count_send_ok (reserved) == 0) {
@@ -1170,6 +1187,7 @@ int totempg_groups_joined_reserve (
 		reserved = 0;
 	}
 
+error_put:
 	hdb_handle_put (&totempg_groups_instance_database, handle);
 
 error_exit:
@@ -1289,6 +1307,16 @@ int totempg_ifaces_get (
 	return (res);
 }
 
+void totempg_event_signal (enum totem_event_type type, int value)
+{
+	totemmrp_event_signal (type, value);
+}
+
+void* totempg_get_stats (void)
+{
+	return &totempg_stats;
+}
+
 int totempg_crypto_set (
 	unsigned int type)
 {
@@ -1343,3 +1371,9 @@ int totempg_my_family_get (void)
 {
 	return (totemmrp_my_family_get());
 }
+extern void totempg_service_ready_register (
+	void (*totem_service_ready) (void))
+{
+	totemmrp_service_ready_register (totem_service_ready);
+}
+
