@@ -136,8 +136,9 @@ enum cs_flow_control_state {
 #endif /* COROSYNC_FLOW_CONTROL_STATE */
 
 enum cs_sync_mode {
-	CS_SYNC_V1	= 0,
-	CS_SYNC_V2	= 1
+	CS_SYNC_V1	 = 0,
+	CS_SYNC_V2	 = 1,
+	CS_SYNC_V1_APIV2 = 2
 };
 
 typedef enum {
@@ -627,6 +628,11 @@ struct corosync_api_v1 {
 		objdb_value_types_t *type);
 
 	void *(*totem_get_stats)(void);
+
+	int (*schedwrk_create_nolock) (
+		hdb_handle_t *handle,
+		int (schedwrk_fn) (const void *),
+		const void *context);
 };
 
 #define SERVICE_ID_MAKE(a,b) ( ((a)<<16) | (b) )
@@ -647,6 +653,11 @@ struct corosync_service_engine_iface_ver0 {
         struct corosync_service_engine *(*corosync_get_service_engine_ver0) (void);
 };
 
+typedef void (*sync_init_v1_fn_t) (
+		const unsigned int *member_list,
+		size_t member_list_entries,
+		const struct memb_ring_id *ring_id) ;
+
 struct corosync_service_engine {
 	const char *name;
 	unsigned short id;
@@ -673,10 +684,7 @@ struct corosync_service_engine {
 		const unsigned int *joined_list, size_t joined_list_entries,
 		const struct memb_ring_id *ring_id);
 	enum cs_sync_mode sync_mode;
-	void (*sync_init) (
-		const unsigned int *member_list,
-		size_t member_list_entries,
-		const struct memb_ring_id *ring_id);
+	sync_init_v1_fn_t sync_init;
 	int (*sync_process) (void);
 	void (*sync_activate) (void);
 	void (*sync_abort) (void);
