@@ -1,12 +1,11 @@
-/*
- * Copyright (c) 2004 MontaVista Software, Inc.
+/* * Copyright (c) 2004 MontaVista Software, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -34,9 +33,14 @@
 #ifndef COROSYNC_EVS_H_DEFINED
 #define COROSYNC_EVS_H_DEFINED
 
-#include <sys/types.h>
+#include <inttypes.h>
 #include <netinet/in.h>
 #include <corosync/corotypes.h>
+#include <corosync/hdb.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /**
  * @defgroup corosync Other API services provided by corosync
@@ -59,25 +63,27 @@ typedef enum {
 #define TOTEMIP_ADDRLEN (sizeof(struct in6_addr))
 
 /** These are the things that get passed around */
-struct evs_address {
-	unsigned int nodeid;
-	unsigned short family;
-	unsigned char addr[TOTEMIP_ADDRLEN];
-};
-
 struct evs_group {
 	char key[32];
 };
 
+struct evs_ring_id {
+	unsigned int nodeid;
+	unsigned long long seq;
+};
+
 typedef void (*evs_deliver_fn_t) (
+	hdb_handle_t handle, 
 	unsigned int nodeid,
-	void *msg,
-	int msg_len);
+	const void *msg,
+	size_t msg_len);
 
 typedef void (*evs_confchg_fn_t) (
-	unsigned int *member_list, int member_list_entries,
-	unsigned int *left_list, int left_list_entries,
-	unsigned int *joined_list, int joined_list_entries);
+	hdb_handle_t handle, 
+	const unsigned int *member_list, size_t member_list_entries,
+	const unsigned int *left_list, size_t left_list_entries,
+	const unsigned int *joined_list, size_t joined_list_entries,
+	const struct evs_ring_id *ring_id);
 
 typedef struct {
 	evs_deliver_fn_t evs_deliver_fn;
@@ -107,6 +113,17 @@ cs_error_t evs_fd_get (
 	evs_handle_t handle,
 	int *fd);
 
+/*
+ * Get and set contexts for a EVS handle
+ */
+cs_error_t evs_context_get (
+	evs_handle_t handle,
+	void **context);
+
+cs_error_t evs_context_set (
+	evs_handle_t handle,
+	void *context);
+
 /*
  * Dispatch messages and configuration changes
  */
@@ -122,16 +139,16 @@ cs_error_t evs_dispatch (
  */
 cs_error_t evs_join (
 	evs_handle_t handle,
-	struct evs_group *groups,
-	int group_cnt);
+	const struct evs_group *groups,
+	size_t group_cnt);
 
 /*
  * Leave one or more groups
  */
 cs_error_t evs_leave (
 	evs_handle_t handle,
-	struct evs_group *groups,
-	int group_cnt);
+	const struct evs_group *groups,
+	size_t group_cnt);
 
 /*
  * Multicast to groups joined with evs_join.
@@ -141,8 +158,8 @@ cs_error_t evs_leave (
 cs_error_t evs_mcast_joined (
 	evs_handle_t handle,
 	evs_guarantee_t guarantee,
-	struct iovec *iovec,
-	int iov_len);
+	const struct iovec *iovec,
+	unsigned int iov_len);
 
 /*
  * Multicast to specified groups.
@@ -152,10 +169,10 @@ cs_error_t evs_mcast_joined (
 cs_error_t evs_mcast_groups (
 	evs_handle_t handle,
 	evs_guarantee_t guarantee,
-	struct evs_group *groups,
-	int group_cnt,
-	struct iovec *iovec,
-	int iov_len);
+	const struct evs_group *groups,
+	size_t group_cnt,
+	const struct iovec *iovec,
+	unsigned int iov_len);
 
 /*
  * Get membership information from evs
@@ -164,6 +181,10 @@ cs_error_t evs_membership_get (
 	evs_handle_t handle,
 	unsigned int *local_nodeid,
 	unsigned int *member_list,
-	unsigned int *member_list_entries);
+	size_t *member_list_entries);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* COROSYNC_EVS_H_DEFINED */
