@@ -1,13 +1,12 @@
 /*
- * Copyright (C) 2006 Red Hat, Inc.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -33,8 +32,8 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef AIS_MAR_GEN_H_DEFINED
-#define AIS_MAR_GEN_H_DEFINED
+#ifndef MAR_GEN_H_DEFINED
+#define MAR_GEN_H_DEFINED
 
 #ifndef COROSYNC_SOLARIS
 #include <stdint.h>
@@ -101,14 +100,15 @@ typedef struct {
 	mar_uint8_t value[CS_MAX_NAME_LENGTH] __attribute__((aligned(8)));
 } mar_name_t;
 
-static inline char *get_mar_name_t (mar_name_t *name) {
-        return ((char *)name->value);
+static inline const char *get_mar_name_t (const mar_name_t *name) {
+        return ((const char *)name->value);
 }
 
-static inline int mar_name_match(mar_name_t *name1, mar_name_t *name2)
+static inline int mar_name_match(const mar_name_t *name1, const mar_name_t *name2)
 {
         if (name1->length == name2->length) {
-                return ((strncmp ((char *)name1->value, (char *)name2->value,
+                return ((strncmp ((const char *)name1->value,
+				  (const char *)name2->value,
                         name1->length)) == 0);
         }
         return 0;
@@ -122,7 +122,7 @@ static inline void swab_mar_name_t (mar_name_t *to_swab)
 
 static inline void marshall_from_mar_name_t (
 	cs_name_t *dest,
-	mar_name_t *src)
+	const mar_name_t *src)
 {
 	dest->length = src->length;
 	memcpy (dest->value, src->value, CS_MAX_NAME_LENGTH);
@@ -130,7 +130,7 @@ static inline void marshall_from_mar_name_t (
 
 static inline void marshall_to_mar_name_t (
 	mar_name_t *dest,
-	cs_name_t *src)
+	const cs_name_t *src)
 {
 	dest->length = src->length;
 	memcpy (dest->value, src->value, CS_MAX_NAME_LENGTH);
@@ -177,4 +177,11 @@ static inline void swab_mar_size_t (mar_size_t *to_swab)
 {
 	swab_mar_uint64_t (to_swab);
 }
-#endif /* AIS_MAR_GEN_TYPES_H_DEFINED */
+
+static inline void swab_coroipc_request_header_t (coroipc_request_header_t *to_swab)
+{
+	swab_mar_int32_t (&to_swab->size);
+	swab_mar_int32_t (&to_swab->id);
+}
+
+#endif /* MAR_GEN_H_DEFINED */
