@@ -35,17 +35,19 @@
 #ifndef TOTEM_H_DEFINED
 #define TOTEM_H_DEFINED
 #include "totemip.h"
+#include <corosync/hdb.h>
 
-#ifndef MESSAGE_SIZE_MAX
-#define MESSAGE_SIZE_MAX	1024*1024 /* (1MB) */
-#endif /* MESSAGE_SIZE_MAX */
-
-#ifndef MESSAGE_QUEUE_MAX
-#define MESSAGE_QUEUE_MAX MESSAGE_SIZE_MAX / totem_config->net_mtu
-#endif /* MESSAGE_QUEUE_MAX */
-
+#ifdef HAVE_SMALL_MEMORY_FOOTPRINT
+#define PROCESSOR_COUNT_MAX	16
+#define MESSAGE_SIZE_MAX	1024*64
+#define MESSAGE_QUEUE_MAX	512
+#else
 #define PROCESSOR_COUNT_MAX	384
-#define FRAME_SIZE_MAX		9000
+#define MESSAGE_SIZE_MAX	1024*1024 /* (1MB) */
+#define MESSAGE_QUEUE_MAX	MESSAGE_SIZE_MAX / totem_config->net_mtu
+#endif /* HAVE_SMALL_MEMORY_FOOTPRINT */
+
+#define FRAME_SIZE_MAX		10000
 #define TRANSMITS_ALLOWED	16
 #define SEND_THREADS_MAX	16
 #define INTERFACE_MAX		2
@@ -161,6 +163,8 @@ struct totem_config {
 
 	int crypto_crypt_type;
 	int crypto_sign_type;
+
+	int transport_number;
 };
 
 #define TOTEM_CONFIGURATION_TYPE
@@ -175,11 +179,83 @@ enum totem_callback_token_type {
 	TOTEM_CALLBACK_TOKEN_SENT = 2
 };
 
+enum totem_event_type {
+	TOTEM_EVENT_DELIVERY_CONGESTED,
+	TOTEM_EVENT_NEW_MSG,
+};
+
 #define MEMB_RING_ID
 struct memb_ring_id {
 	struct totem_ip_address rep;
 	unsigned long long seq;
 } __attribute__((packed));
 
+typedef struct {
+	hdb_handle_t handle;
+	int is_dirty;
+	time_t last_updated;
+} totem_stats_header_t;
+
+typedef struct {
+	totem_stats_header_t hdr;
+	uint32_t iface_changes;
+} totemnet_stats_t;
+
+typedef struct {
+	totem_stats_header_t hdr;
+	totemnet_stats_t *net;
+	char *algo_name;
+} totemrrp_stats_t;
+
+
+typedef struct {
+	uint32_t rx;
+	uint32_t tx;
+	int backlog_calc;
+} totemsrp_token_stats_t;
+
+typedef struct {
+	totem_stats_header_t hdr;
+	totemrrp_stats_t *rrp;
+	uint64_t orf_token_tx;
+	uint64_t orf_token_rx;
+	uint64_t memb_merge_detect_tx;
+	uint64_t memb_merge_detect_rx;
+	uint64_t memb_join_tx;
+	uint64_t memb_join_rx;
+	uint64_t mcast_tx;
+	uint64_t mcast_retx;
+	uint64_t mcast_rx;
+	uint64_t memb_commit_token_tx;
+	uint64_t memb_commit_token_rx;
+	uint64_t token_hold_cancel_tx;
+	uint64_t token_hold_cancel_rx;
+	uint64_t operational_entered;
+	uint64_t operational_token_lost;
+	uint64_t gather_entered;
+	uint64_t gather_token_lost;
+	uint64_t commit_entered;
+	uint64_t commit_token_lost;
+	uint64_t recovery_entered;
+	uint64_t recovery_token_lost;
+	uint64_t consensus_timeouts;
+	uint64_t rx_msg_dropped;
+
+	int earliest_token;
+	int latest_token;
+#define TOTEM_TOKEN_STATS_MAX 100
+	totemsrp_token_stats_t token[TOTEM_TOKEN_STATS_MAX];
+
+} totemsrp_stats_t;
+
+typedef struct {
+	totem_stats_header_t hdr;
+	totemsrp_stats_t *srp;
+} totemmrp_stats_t;
+
+typedef struct {
+	totem_stats_header_t hdr;
+	totemmrp_stats_t *mrp;
+} totempg_stats_t;
 
 #endif /* TOTEM_H_DEFINED */
