@@ -362,6 +362,14 @@ cs_error_t quorum_dispatch (
 			error = CS_OK;
 			goto error_put;
 		}
+		if (error == CS_ERR_TRY_AGAIN) {
+			error = CS_OK;
+			if (dispatch_types == CPG_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
+			} else {
+				continue; /* next poll */
+			}
+		}
 		if (error != CS_OK) {
 			goto error_put;
 		}
@@ -400,15 +408,9 @@ cs_error_t quorum_dispatch (
 
 		/*
 		 * Determine if more messages should be processed
-		 * */
-		switch (dispatch_types) {
-		case CS_DISPATCH_ONE:
+		 */
+		if (dispatch_types == CS_DISPATCH_ONE) {
 			cont = 0;
-			break;
-		case CS_DISPATCH_ALL:
-			break;
-		case CS_DISPATCH_BLOCKING:
-			break;
 		}
 	} while (cont);
 
