@@ -39,8 +39,8 @@ const char *sa_error_list[] = {
 
 int get_sa_error(cs_error_t error, char *str, int len)
 {
-	if (error < CS_OK || 
-			error > CS_ERR_NO_SECTIONS || 
+	if (error < CS_OK ||
+			error > CS_ERR_NO_SECTIONS ||
 					len < strlen(sa_error_list[error])) {
 			errno = EINVAL;
 		return -1;
@@ -59,7 +59,7 @@ static char test_result[256];
         if (result == expected) {
                 return ("PASSED");
         } else {
-                sprintf (test_result,
+                snprintf (test_result, sizeof(test_result),
                         "FAILED expected %s got %s",
 			get_sa_error_b(expected), get_sa_error_b(result));
                 return (test_result);
